@@ -3,7 +3,7 @@
  * This code is part of GOsa (http://www.gosa-project.org)
  * Copyright (C) 2003-2008 GONICUS GmbH
  *
- * ID: $$Id: main.php 18935 2010-07-06 05:55:51Z hickert $$
+ * ID: $$Id: main.php 20708 2011-04-12 08:41:49Z hickert $$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 /* Save start time */
 $start = microtime();
 
+// Will be used in the "stats" plugin later, to be able calculate the elapsed render time.
+$overallRenderTimer = microtime(TRUE);
+
 /* Basic setup, remove eventually registered sessions */
 require_once ("../include/php_setup.inc");
 require_once ("functions.inc");
@@ -35,12 +38,23 @@ $domain = 'messages';
 bindtextdomain($domain, LOCALE_DIR);
 textdomain($domain);
 
+
 /* Remember everything we did after the last click */
 session::start();
 session::set('errorsAlreadyPosted',array());
 session::global_set('runtime_cache',array());
 session::set('limit_exceeded',FALSE);
 
+// Count number of page reloads 
+if(!session::is_set('clicks')){
+    session::set('clicks', 0);
+}
+$clicks = session::get('clicks');
+$clicks ++ ;
+session::set('clicks', $clicks);
+
+pathNavigator::clear();
+
 if ($_SERVER["REQUEST_METHOD"] == "POST"){
   @DEBUG (DEBUG_POST, __LINE__, __FUNCTION__, __FILE__, $_POST, "_POST");
 }
@@ -62,9 +76,10 @@ if ($_SERVER['REMOTE_ADDR'] != $ui->ip){
 }
 $config= session::global_get('config');
 $config->check_and_reload();
+$config->configRegistry->reload();
 
 /* Enable compressed output */
-if ($config->get_cfg_value("sendCompressedOutput") == "true"){
+if ($config->get_cfg_value("core","sendCompressedOutput") == "true"){
   ob_start("ob_gzhandler");
 }
 
@@ -74,7 +89,7 @@ if(session::global_get('_LAST_PAGE_REQUEST') == ""){
 }else{
 
   /* check GOsa.conf for defined session lifetime */
-  $max_life= $config->get_cfg_value("sessionLifetime", 60*60*2);
+  $max_life= $config->get_cfg_value("core","sessionLifetime");
 
   /* get time difference between last page reload */
   $request_time = (time()- session::global_get('_LAST_PAGE_REQUEST'));
@@ -95,7 +110,8 @@ if(session::global_get('_LAST_PAGE_REQUEST') == ""){
 @DEBUG (DEBUG_CONFIG, __LINE__, __FUNCTION__, __FILE__, $config->data, "config");
 
 /* Set template compile directory */
-$smarty->compile_dir= $config->get_cfg_value("templateCompileDirectory", '/var/spool/gosa');
+$smarty->compile_dir= $config->get_cfg_value("core","templateCompileDirectory");
+$smarty->error_unassigned= true;
 
 /* Set default */
 $reload_navigation = false;
@@ -126,6 +142,13 @@ setlocale(LC_ALL, $lang);
 $GLOBALS['t_language']= $lang;
 $GLOBALS['t_gettext_message_dir'] = $BASE_DIR.'/locale/';
 
+// Validate LDAP schema if not done already
+if( $config->boolValueIsTrue('core','schemaCheck') && 
+    !$config->configRegistry->schemaCheckFinished() && 
+    !$config->configRegistry->validateSchemata($force=FALSE,$disableIncompatiblePlugins=TRUE)){
+    $config->configRegistry->displayRequirementErrors();
+}
+
 /* Check if the config is up to date */
 $config->check_config_version();
 
@@ -145,8 +168,8 @@ if (!session::global_is_set('plist')){
         require_once("$BASE_DIR/$path");
       } else {
         msg_dialog::display(_("Fatal error"),
-            sprintf(_("Cannot locate file '%s' - please run '%s' to fix this"),
-              "$BASE_DIR/$path", "<b>update-gosa</b>"), FATAL_ERROR_DIALOG);
+            sprintf(_("Cannot locate file %s - please run %s to fix this"),
+              bold("$BASE_DIR/$path"), bold("update-gosa")), FATAL_ERROR_DIALOG);
         exit;
       }
     }
@@ -162,10 +185,10 @@ if (!session::global_is_set('plist')){
 $plist= session::global_get('plist');
 
 /* Check for register globals */
-if (isset($global_check) && $config->get_cfg_value("forceglobals") == "true"){
+if (isset($global_check) && $config->boolValueIsTrue("core","forceGlobals")){
   msg_dialog::display(
             _("PHP configuration"),
-            _("FATAL: Register globals is on. GOsa will refuse to login unless this is fixed by an administrator."),
+            _("Fatal error: Register globals is active. Please fix this in order to continue."),
             FATAL_ERROR_DIALOG);
 
   new log("security","login","",array(),"Register globals is on. For security reasons, this should be turned off.") ;
@@ -180,24 +203,23 @@ if (session::global_is_set('plugin_dir')){
   $old_plugin_dir= "";
 }
 
-/* reload navigation if language changed*/  
-if($reload_navigation){
-  $plist->menu="";
-}
+// Generate menus 
 $plist->gen_headlines();
 $plist->gen_menu();
+$plist->genPathMenu();
 
 /* check if we are using account expiration */
 $smarty->assign("hideMenus", FALSE);
-if ($config->get_cfg_value("handleExpiredAccounts") == "true"){
+if ($config->boolValueIsTrue("core","handleExpiredAccounts")){
     $expired= ldap_expired_account($config, $ui->dn, $ui->username);
+ 
     if ($expired == POSIX_WARN_ABOUT_EXPIRATION && !session::is_set('POSIX_WARN_ABOUT_EXPIRATION__DONE')){
 
         // The users password is about to xpire soon, display a warning message.
         new log("security","gosa","",array(),"password for user \"$ui->username\" is about to expire") ;
         msg_dialog::display(_("Password change"), _("Your password is about to expire, please change your password!"), INFO_DIALOG);
         session::set('POSIX_WARN_ABOUT_EXPIRATION__DONE', TRUE);
-    
+
     } elseif ($expired == POSIX_FORCE_PASSWORD_CHANGE){
 
         // The password is expired, we are now going to enforce a new one from the user.
@@ -206,22 +228,20 @@ if ($config->get_cfg_value("handleExpiredAccounts") == "true"){
         $smarty->assign("hideMenus", TRUE);
         $plug = (isset($_GET['plug'])) ? $_GET['plug'] : null;
 
-        // Search for the 'password' class and set its id as active plug.
-        foreach ($plist->dirlist as $key => $value){
-            if (preg_match("/\bpassword\b/i",$value)){
-                if($plug != $key) {
-                    $_GET['plug'] = $key;
-                }
-                break;
-            }
+        // Detect password plugin id:
+        $passId =  array_search('password', $plist->pluginList);
+        if($passId !== FALSE){
+            $_GET['plug'] = $passId;
         }
     }
 }
 
-
+$smarty->assign("noMenuMode", count($plist->getRegisteredMenuEntries()) == 0);
 if (isset($_GET['plug']) && $plist->plugin_access_allowed($_GET['plug'])){
   $plug= validate($_GET['plug']);
   $plugin_dir= $plist->get_path($plug);
+  $plugin= $plist->get_class($plug);
+  session::global_set('currentPlugin',$plugin);
   session::global_set('plugin_dir',$plugin_dir);
   if ($plugin_dir == ""){
     new log("security","gosa","",array(),"main.php called with invalid plug parameter \"$plug\"") ;
@@ -229,12 +249,19 @@ if (isset($_GET['plug']) && $plist->plugin_access_allowed($_GET['plug'])){
     exit;
   }
 } else {
-
-  /* set to welcome page as default plugin */
-  session::global_set('plugin_dir',"welcome");
-  $plugin_dir= "$BASE_DIR/plugins/generic/welcome";
+    session::global_set('plugin_dir',"welcome");
+    session::global_set('currentPlugin','welcome');
+    $plugin_dir= "$BASE_DIR/plugins/generic/welcome";
 }
 
+// Display the welcome page for admins (iconmenu) and an info page for those 
+//  who are not allowed to adminstrate anything (user)
+if(count($plist->getRegisteredMenuEntries()) == 0 && session::global_get('currentPlugin') == "welcome"){
+    session::global_set('plugin_dir',"infoPage");
+    session::global_set('currentPlugin','welcome');
+    $plugin_dir= "$BASE_DIR/plugins/generic/infoPage";
+} 
+
 /* Handle plugin locks.
     - Remove the plugin from session if we switched to another. (cleanup) 
     - Remove all created locks if "reset" was posted.
@@ -305,9 +332,12 @@ if (isset($_GET['reset'])){
 }
 
 /* show web frontend */
+$smarty->assign ("title","GOsa");
+$smarty->assign ("logo", image(get_template_path("images/logo.png")));
+$smarty->assign ("logoutimage", get_template_path("images/btn-logout.png"));
 $smarty->assign ("date", date("l, dS F Y H:i:s O"));
 $smarty->assign ("lang", preg_replace('/_.*$/', '', $lang));
-$smarty->assign ("must", "<font class=\"must\">*</font>");
+$smarty->assign ("must", "<span class='required'>*</span>");
 if (isset($plug)){
   $plug= "?plug=$plug";
 } else {
@@ -321,11 +351,14 @@ if (session::global_get('js')==FALSE){
   $smarty->assign("help_method"," onclick=\"return popup('helpviewer.php$plug','GOsa help');\"");
 }
 
+
+$loggedin = sprintf(_("You're logged in as %s"), 
+    "<span>".$ui->cn." [".$ui->username."] / ".$config->current['NAME']."</span> &nbsp;");
 if($ui->ignore_acl_for_current_user()){
-  $smarty->assign ("username", "<font color='#FF0000';>"._("User ACL checks disabled")."</font>&nbsp;".$ui->username);
-}else{
-  $smarty->assign ("username", $ui->username);
+    $loggedin = "<font color='red'>"._("ACLs are disabled")."</font>&nbsp;".$loggedin;
 }
+
+$smarty->assign ("loggedin", $loggedin);
 $smarty->assign ("go_logo", get_template_path('images/go_logo.png'));
 $smarty->assign ("go_base", get_template_path('images/dtree.png'));
 $smarty->assign ("go_home", get_template_path('images/gohome.png'));
@@ -335,11 +368,13 @@ $smarty->assign ("go_corner", get_template_path('images/go_corner.png'));
 $smarty->assign ("go_left", get_template_path('images/go_left.png'));
 $smarty->assign ("go_help", get_template_path('images/help.png'));
 
-$smarty->assign ("menu", $plist->menu);
+/* reload navigation if language changed*/  
+if($reload_navigation){
+  $plist->menu="";
+}
+$smarty->assign ("menu", $plist->gen_menu());
 $smarty->assign ("plug", "$plug");
 
-$smarty->assign("iePngWorkaround", $config->get_cfg_value("iePngWorkaround","false" ) == "true");
-$smarty->assign("usePrototype", "false");
 
 /* React on clicks */
 if ($_SERVER["REQUEST_METHOD"] == "POST"){
@@ -370,8 +405,8 @@ if (is_file("$plugin_dir/main.inc")){
   require ("$plugin_dir/main.inc");
 } else {
   msg_dialog::display(
-      _("Plugin"),
-      sprintf(_("FATAL: Cannot find any plugin definitions for plugin '%s'!"), $plug),
+      _("Plug-in"),
+      sprintf(_("Fatal error: Cannot find any plugin definitions for plugin %s!"), bold($plug)),
       FATAL_ERROR_DIALOG);
   exit();
 }
@@ -379,14 +414,16 @@ if (is_file("$plugin_dir/main.inc")){
 
 /* Print_out last ErrorMessage repeated string. */
 $smarty->assign("msg_dialogs", msg_dialog::get_dialogs());
+$smarty->assign ("pathMenu", $plist->genPathMenu());
 $smarty->assign("contents", $display);
+$smarty->assign("sessionLifetime", $config->get_cfg_value('core','sessionLifetime'));
 
 /* If there's some post, take a look if everything is there... */
 if (isset($_POST) && count($_POST)){
   if (!isset($_POST['php_c_check'])){
     msg_dialog::display(
             _("Configuration Error"),
-            sprintf(_("FATAL: not all POST variables have been transfered by PHP - please inform your administrator!")),
+            sprintf(_("Fatal error: not all POST variables have been transfered by PHP - please inform your administrator!")),
             FATAL_ERROR_DIALOG);
     exit();
   }
@@ -447,7 +484,7 @@ if(isset($_COOKIE['GOsa_Filter_Settings'])){
 }
 
 /* Save filters? */
-if($config->get_cfg_value("storeFilterSettings") == "true"){
+if($config->get_cfg_value("core","storeFilterSettings") == "true"){
   $cookie_vars = array("MultiDialogFilters","CurrentMainBase");
   foreach($cookie_vars as $var){
     if(session::global_is_set($var)){
